@@ -9,9 +9,18 @@
 #include <glib.h>
 #include <libsoup/soup-message.h>
 
+G_BEGIN_DECLS
+
 typedef struct SoupMessageQueue SoupMessageQueue; 
 
+/**
+ * SoupMessageQueueIter:
+ *
+ * An opaque data structure used to iterate the elements of a
+ * #SoupMessageQueue.
+ **/
 typedef struct {
+	/*< private >*/
 	GList *cur, *next;
 } SoupMessageQueueIter;
 
@@ -34,4 +43,6 @@ void              soup_message_queue_destroy    (SoupMessageQueue     *queue);
 void              soup_message_queue_remove_message (SoupMessageQueue *queue,
 						     SoupMessage      *msg);
 
+G_END_DECLS
+
 #endif /* SOUP_MESSAGE_QUEUE_H */
